/*
 *  This Loader Module is written by hcs
 *
 */

/*
        L O A D E R  for Project 64 v1.4 Save State
*/

#define __MSDOS__

#include "..\idaldr.h"
#include <exehdr.h>
#include <queue.hpp>
#include "pj64.h"
#include <stddef.h>   //offsetof

//--------------------------------------------------------------------------
//
//      check input file format. if recognized, then return 1
//      and fill 'fileformatname'.
//      otherwise return 0
//
int accept_file(FILE *fp, char fileformatname[MAX_FILE_FORMAT_NAME], int n)
{
  ulong fl;
  pj64_hdr header;

  if(n) return(0);

  if(qfread(fp, &header, sizeof(header)) != sizeof(header) ||
     header.signature != PJ64_SIGNATURE) return(0);
  fl = efilelength(fp);
  if (fl != 0x75c+header.ramsize+0x2000) return(0);

  strncpy(fileformatname, "Project 64 v1.4 Save State", MAX_FILE_FORMAT_NAME);
  return(1);
}

//--------------------------------------------------------------------------
//
//      load file into the database.
//
void load_file(FILE *fp, ushort /*neflag*/, const char * /*fileformatname*/)
{
  pj64_hdr header;
  unsigned char * RAM;
  uint32 c;

  set_processor_type("mipsb", SETPROC_ALL|SETPROC_FATAL);
  eread(fp, &header, sizeof(header));

  // RAM segment
  add_segm(0,0x80000000,0x80000000+header.ramsize,"RAM",NULL);

  msg("Loading...\n");

  if (!(RAM=(unsigned char*)qalloc(header.ramsize))) qexit(1);

  eseek(fp, 0x75c);
  eread(fp, RAM, header.ramsize);

  msg("Byteswapping...\n");

  for (c=0; c < header.ramsize; c+=4) {
	  ((uint32*)RAM)[c/4]=(RAM[c]<<24)|(RAM[c+1]<<16)|(RAM[c+2]<<8)|RAM[c+3];
  }
  
  mem2base(RAM,0x80000000,0x80000000+header.ramsize,0);

  qfree(RAM);

  // a starting point for analysis (since we know there's code there)
  autoMark(header.PC,AU_CODE);

  plan_to_apply_idasgn("LIBULTRA");
  plan_to_apply_idasgn("LIBULTRA2");
  plan_to_apply_idasgn("LIBGULTRA");
}

//----------------------------------------------------------------------
//
//      LOADER DESCRIPTION BLOCK
//
//----------------------------------------------------------------------

extern "C" loader_t LDSC = {
  IDP_INTERFACE_VERSION,
  0,                            // loader flags
//
//      check input file format. if recognized, then return 1
//      and fill 'fileformatname'.
//      otherwise return 0
//
  accept_file,
//
//      load file into the database.
//
  load_file,
//
//      create output file from the database.
//      this function may be absent.
//
  NULL,
};
